import java.awt.*;
import java.lang.*;
import java.awt.event.*;
import java.util.*;
import java.awt.geom.*; 
import javax.swing.*; 
import javax.swing.event.*;

/**
 * Implementa el Panel de dibujo. Incluye los mtodos para
 * atender eventos de mouse y para relizar acciones y dibujos.
 */
public class DrawingPanel extends JPanel
{

//Atributos
/**
 * Almacena las figuras creadas por el usuario
 */
private ArrayList figuras;
/**
 * Punto del primer click.
 */
private Point2D puntoInicial;
/**
 * Punto de Comparacin. Usualmente corresponde al primer
 * punto antes de comenzar unn arrastre o drag.
 */
private Point2D puntoComp;
/**
 * Punto Actual. Es el punto en que el mouse se encuentra
 * en este momento.
 */
private Point2D puntoAct;
/**
 * Identifica el click con botn izquierdo.
 */
private boolean oneClick;
/**
 * Identifica click con botn izquierdo.
 */
private boolean twoClick;
/**
 * Identifica un doble click.
 */
private boolean otherClick;
/**
 * Almacena Circulo mientras se dibuja, antes de pasarlo
 * al arreglo de Figuras.
 */
private Ellipse2D figsC;
/**
 * Almacena Rectngulo mientras se dibuja, antes de pasarlo
 * al arreglo de Figuras.
 */
private Rectangle2D figsR;
/**
 * Almacena la lnea mientras se dibuja, antes de pasarlo
 * al arreglo de Figuras.
 */
private Line2D figsP;
/**
 * Contiene la actual transformacin de Rotacin.
 */
private AffineTransform tformR = new AffineTransform();
/**
 * Contiene la actual transformacin de Traslacin.
 */
private AffineTransform tformT = new AffineTransform();
/**
 * Contiene la actual transformacin de Escalamiento.
 */
private AffineTransform tformS = new AffineTransform();
/**
 * Evita dibujar una figura cuando se estn realizaando
 * acciones de manipulacin.
 */
private boolean dontDraw;
/**
 * Identifica botn oprimido del mouse.
 */
private boolean pressed;
/**
 * Identifica botn soltado de mouse.
 */
private boolean released;
/**
 * Identifica cambio de tamao de panel de dibujo.
 */
private boolean evento;
/**
 * Angulo de Rotacin actual
 */
private double angulo;
/**
 * Factor de Escalamiento actual.
 */
private double factor;
/**
 * Distancia actual desde punto inicial a Punto Actual.
 */
private double distAct;
/**
 * Distancia actual desde punto inicial a Punto de Comparacin.
 */
private double distComp;

private final int blanco = 255;

//Constructor
/**
 * Constructor. Crea el panel de dibujo y agrega los 
 * manejadores de eventos de mouse.
 */
 public DrawingPanel()
 {
 figuras = new ArrayList();
 puntoInicial = null;
 figsC = new Ellipse2D.Double();
 figsR = new Rectangle2D.Double();
 figsP = new Line2D.Double();
 tformR.setToIdentity();
 tformT.setToIdentity();
 tformS.setToIdentity();
 oneClick = false;
 twoClick = false;
 otherClick = false;
 dontDraw = false;
 evento = false;
 angulo = 0;

 setOpaque(true);
 /* Un bug de Java no permite poner un borde para un panel en que se
  * va a declarar el paintComponent(). Si se des-comenta la linea de setBorder
  * ese borde tambin sufrir las AffineTransforms.
  * setBorder(BorderFactory.createLineBorder(Color.black));
  */
 setBackground(new Color(blanco, blanco, blanco));
 
 addMouseListener(new MouseHandler());
 addMouseMotionListener(new MouseMotionHandler());
 addComponentListener(new ComponentHandler());
 }
 
//Metodos
/**
 * Implementa el mtodo adecuado para el pintado de la pantalla.
 * Se incluy el cdigo para implementar las manipulaciones, adems
 * de slo dibujar las figuras.
 * @param g El componente grfico <tt>g</tt>.
 */
public void paintComponent(Graphics g)
   { super.paintComponent(g);
     Graphics2D g2 = (Graphics2D)g;
   /*g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, 
    * RenderingHints.VALUE_ANTIALIAS_ON);
    */
    if (!evento) 
    { if (ButtonHandler.whatToDo() == "R") 
     { if (pressed)
       { g2.transform(tformR); pressed = false; }
       g2.rotate(angulo, puntoInicial.getX(), puntoInicial.getY()); 
       dontDraw = true; tformR = g2.getTransform(); }
      if (ButtonHandler.whatToDo() == "T")
     { if (pressed)
       { g2.transform(tformT); pressed = false; }
       g2.translate(puntoAct.getX() - puntoComp.getX(), 
          puntoAct.getY() - puntoComp.getY()); 
       dontDraw = true; tformT = g2.getTransform(); }
      if (ButtonHandler.whatToDo() == "S")
      { if (pressed)
       { g2.transform(tformS); pressed = false; }
       g2.translate(-factor * puntoInicial.getX() + puntoInicial.getX(), 
          -factor * puntoInicial.getY() + puntoInicial.getY());
       g2.scale(factor, factor);
       dontDraw = true; tformS = g2.getTransform(); }
       }
      if ((ButtonHandler.whatToDo() != "R") 
           | (released && ButtonHandler.whatToDo() == "R"))
      { g2.transform(tformR); released = false; }
      if ((ButtonHandler.whatToDo() != "T") 
           | (released && ButtonHandler.whatToDo() == "T"))
      { g2.transform(tformT); released = false; }
      if ((ButtonHandler.whatToDo() != "S") 
           | (released && ButtonHandler.whatToDo() == "S"))
      { g2.transform(tformS); released = false; }
    for (int i = 0; i < figuras.size(); i++)
         g2.draw((Shape)figuras.get(i));
      if (!dontDraw)
      { g2.draw((Ellipse2D.Double)figsC); g2.draw((Rectangle2D.Double)figsR);
        g2.draw((Line2D.Double)figsP); }
     }
  
/**
 * Clase para manejar cambios en el drawing Panel
 */
private class ComponentHandler extends ComponentAdapter
{
  
/**
 * Detecta una modificacion en el tamao del Panel para manejar
 * mejor el proceso de pintado
 * @param c El <tt>componente</tt>
 */     
      public void componentResized(ComponentEvent c)
      {
        evento = true;
      }
  }

/**
 * Clase para manejar eventos de mouse
 */
private class MouseHandler extends MouseAdapter
{ private final int tres = 3;
/**
 * Atiende eventos de click de mouse
 * @param event El <tt>evento</tt> de click.
 */      
      public void mouseClicked(MouseEvent event)
      {
         evento = false;   
         //Decodifica botones de Mouse
         if (event.getButton() == 1)
         {
            puntoInicial = event.getPoint();
          if (event.getClickCount() >= 2)
            {
                     otherClick = true;
                     oneClick = true;
                     twoClick = false; }
          else
          { oneClick = true;
            twoClick = false;
            otherClick = false; }
         }
         else if (event.getButton() == tres) 
         {  oneClick = false;
            twoClick = true;
            otherClick = false; }
         else
         { oneClick = false; twoClick = false; otherClick = false; }
      }
/**
 * Atiende evento de presin de un botn mouse.
 * @param event El <tt>evento</tt> de presionar un botn.
 */
     public void mousePressed(MouseEvent event)
      {
       pressed = true;
      }
/**
 * Atiende evento de soltar un botn de mouse.
 * @param event El <tt>evento</tt> de soltar un botn.
 */ 
     public void mouseReleased(MouseEvent event)
     {
      released = true;
     }
    }
 
/**
 * Clase para manejar movimientos de mouse.
 */ 
 private class MouseMotionHandler implements MouseMotionListener
 {
/**
 * Maneja eventos relacionados con movimiento del mouse.
 * Dentro de este mtodo se encuentra el cdigo para dibujar
 * las distintas figuras.
 * Se invoca al mtodo repaint() para dibujar una forma.
 * @param event El <tt>evento</tt> de mover el mouse.
 */
      public void mouseMoved(MouseEvent event)
      { if (ButtonHandler.whichToDraw() == "C")
        { if (oneClick)
         { double rect; double coord; double size;
          double rectWidth = event.getX();
          double rectHeight = event.getY();
          double x = puntoInicial.getX();
          double y = puntoInicial.getY();
       if (rectWidth < rectHeight) rect = rectWidth;
          else rect = rectHeight;
       if (x < y) coord = x;
          else coord = y;
        size = Math.abs(rect - coord);
        figsC = new Ellipse2D.Double(x, y, size, size);
        dontDraw = false; repaint(); }
         if (twoClick)
         { figuras.add(figsC); }
       }
      if (ButtonHandler.whichToDraw() == "R")
        { if (oneClick)
          { double rectWidth = event.getX();
            double rectHeight = event.getY();
            double x = puntoInicial.getX();
            double y = puntoInicial.getY();
       figsR = new Rectangle2D.Double(x, y, java.lang.Math.abs(rectWidth - x), 
            java.lang.Math.abs(rectHeight - y));
       dontDraw = false; repaint(); }
       if (twoClick)
         { figuras.add(figsR); }
       }
       if (ButtonHandler.whichToDraw() == "P")
       { if (oneClick && !otherClick)
         { Point2D puntoFinal = event.getPoint();
           figsP = new Line2D.Double(puntoInicial, puntoFinal);
           dontDraw = false; repaint();  }
        if (twoClick)
        { figuras.add(figsP); }
        if (otherClick)
        { figuras.add(figsP); otherClick = false; }  
       }
    } 

/**
 * Maneja eventos relacionados con arrastres del mouse.
 * Dentro de este mtodo se encuentra el cdigo para manipular
 * las distintas figuras.
 * Se invoca al mtodo repaint() para introducir una transformacin.
 * @param event El <tt>evento</tt> de mover el mouse.
 */  
   public void mouseDragged(MouseEvent event)
   { if (ButtonHandler.whatToDo() == "R")
      { if (pressed)
         { puntoComp = event.getPoint();
           oneClick = false; }

/* Esta es una forma de obtener el angulo. Se usa la distancia entre
 * una linea y el punto actual, donde la linea va entre el punto de click
 * inicial y el primer punto de arrastre.*/
  Line2D.Double linea = new Line2D.Double(puntoInicial, puntoComp);
  puntoAct = event.getPoint();
  distComp = puntoInicial.distance(puntoAct);
  distAct = linea.ptLineDist(puntoAct);
     if (puntoComp.getY() > puntoAct.getY())
       { angulo = -(distAct / (distComp)) + (2 * Math.PI); } 
     else 
       { angulo = distAct / (distComp); }
 /* angulo = -Math.atan2(puntoAct.getX(), puntoAct.getY());*/
  repaint();
 }

         if (ButtonHandler.whatToDo() == "T")
         { if (pressed)
           { puntoComp = event.getPoint(); }
             puntoAct = event.getPoint();
             repaint();
          }

          if (ButtonHandler.whatToDo() == "S")
         { if (pressed)
           { puntoComp = event.getPoint();
             distComp = puntoInicial.distance(puntoComp);
             oneClick = false; }
           puntoAct = event.getPoint();
           distAct = puntoComp.distance(puntoAct);
           factor = (distAct / distComp);
           repaint();
          }
     }
   }
 
 }
